package com.aliyun.odps.spark.examples.makedata

import com.alibaba.fastjson.JSONObject
import com.alibaba.fastjson.JSON
import com.aliyun.odps.TableSchema
import com.aliyun.odps.data.Record
import com.aliyun.odps.spark.examples.myUtils.{MyDateUtils, MyHdfsUtils, ParamUtils, env}
import examples.extractor.{RankExtractorItemFeatureV2, RankExtractorUserFeatureV2}
import org.apache.hadoop.io.compress.GzipCodec
import org.apache.spark.sql.SparkSession

import java.util
import java.util.{HashMap, Map}
import scala.collection.JavaConversions._
import scala.collection.mutable
import examples.extractor.RankExtractorFeature_20240530
/*
   所有获取不到的特征，给默认值0.
 */

object makedata_13_originData_20240529 {
  def main(args: Array[String]): Unit = {
    val spark = SparkSession
      .builder()
      .appName(this.getClass.getName)
      .getOrCreate()
    val sc = spark.sparkContext

    // 1 读取参数
    val param = ParamUtils.parseArgs(args)
    val tablePart = param.getOrElse("tablePart", "64").toInt
    val partitionPrefix = param.getOrElse("partitionPrefix", "dt={},hh={}")
    val beginStr = param.getOrElse("beginStr", "2023010100")
    val endStr = param.getOrElse("endStr", "2023010123")
    val savePath = param.getOrElse("savePath", "/dw/recommend/model/13_sample_data/")
    val project = param.getOrElse("project", "loghubods")
    val table = param.getOrElse("table", "XXXX")


    // 2 读取odps+表信息
    val odpsOps = env.getODPS(sc)

    // 3 循环执行数据生产
    val timeRange = MyDateUtils.getDateHourRange(beginStr, endStr)
    for (dt_hh <- timeRange) {
      val dt = dt_hh.substring(0, 8)
      val hh = dt_hh.substring(8, 10)
      val partition = s"dt=$dt,hh=$hh"
      println("开始执行partiton:" + partition)
      val odpsData = odpsOps.readTable(project = project,
        table = table,
        partition = partition,
        transfer = func,
        numPartition = tablePart)
        .map(record => {
          // a 视频特征
          val b1: JSONObject = if (record.isNull("b1_feature")) new JSONObject() else
            JSON.parseObject(record.getString("b1_feature"))
          val b2: JSONObject = if (record.isNull("b2_feature")) new JSONObject() else
            JSON.parseObject(record.getString("b2_feature"))
          val b3: JSONObject = if (record.isNull("b3_feature")) new JSONObject() else
            JSON.parseObject(record.getString("b3_feature"))
          val b6: JSONObject = if (record.isNull("b6_feature")) new JSONObject() else
            JSON.parseObject(record.getString("b6_feature"))
          val b7: JSONObject = if (record.isNull("b7_feature")) new JSONObject() else
            JSON.parseObject(record.getString("b7_feature"))

          val b8: JSONObject = if (record.isNull("b8_feature")) new JSONObject() else
            JSON.parseObject(record.getString("b8_feature"))
          val b9: JSONObject = if (record.isNull("b9_feature")) new JSONObject() else
            JSON.parseObject(record.getString("b9_feature"))
          val b10: JSONObject = if (record.isNull("b10_feature")) new JSONObject() else
            JSON.parseObject(record.getString("b10_feature"))
          val b11: JSONObject = if (record.isNull("b11_feature")) new JSONObject() else
            JSON.parseObject(record.getString("b11_feature"))
          val b12: JSONObject = if (record.isNull("b12_feature")) new JSONObject() else
            JSON.parseObject(record.getString("b12_feature"))
          val b13: JSONObject = if (record.isNull("b13_feature")) new JSONObject() else
            JSON.parseObject(record.getString("b13_feature"))
          val b17: JSONObject = if (record.isNull("b17_feature")) new JSONObject() else
            JSON.parseObject(record.getString("b17_feature"))
          val b18: JSONObject = if (record.isNull("b18_feature")) new JSONObject() else
            JSON.parseObject(record.getString("b18_feature"))
          val b19: JSONObject = if (record.isNull("b19_feature")) new JSONObject() else
            JSON.parseObject(record.getString("b19_feature"))

          val featureMap = new util.HashMap[String, Double]()
          val origin_data = List(
            (b1, b2, b3, "b123"), (b1, b6, b7, "b167"),
            (b8, b9, b10, "b8910"), (b11, b12, b13, "b111213"),
            (b17, b18, b19, "b171819")
          )
          for ((b_1, b_2, b_3, prefix1) <- origin_data){
            for (prefix2 <- List(
              "1h", "2h", "3h", "4h", "12h", "1d", "3d", "7d"
            )){
              val exp = if (b_1.isEmpty) 0D else b_1.getIntValue("exp_pv_" + prefix2).toDouble
              val share = if (b_2.isEmpty) 0D else b_2.getIntValue("share_pv_" + prefix2).toDouble
              val returns = if (b_3.isEmpty) 0D else b_3.getIntValue("return_uv_" + prefix2).toDouble
              val f1 = RankExtractorFeature_20240530.calDiv(share, exp)
              val f2 = RankExtractorFeature_20240530.calLog(share)
              val f3 = RankExtractorFeature_20240530.calDiv(returns, exp)
              val f4 = RankExtractorFeature_20240530.calLog(returns)
              val f5 = f3 * f4
              featureMap.put(prefix1 + "_" + prefix2 + "_" + "STR", f1)
              featureMap.put(prefix1 + "_" + prefix2 + "_" + "log(share)", f2)
              featureMap.put(prefix1 + "_" + prefix2 + "_" + "ROV", f3)
              featureMap.put(prefix1 + "_" + prefix2 + "_" + "log(return)", f4)
              featureMap.put(prefix1 + "_" + prefix2 + "_" + "ROV*log(return)", f5)
            }
          }

          val video_info: JSONObject = if (record.isNull("t_v_info_feature")) new JSONObject() else
            JSON.parseObject(record.getString("t_v_info_feature"))
          featureMap.put("total_time", if (video_info.containsKey("total_time")) video_info.getIntValue("total_time").toDouble else 0D)
          featureMap.put("bit_rate", if (video_info.containsKey("bit_rate")) video_info.getIntValue("bit_rate").toDouble else 0D)

          val c1: JSONObject = if (record.isNull("c1_feature")) new JSONObject() else
            JSON.parseObject(record.getString("c1_feature"))
          featureMap.put("playcnt_6h", if (c1.containsKey("playcnt_6h")) c1.getIntValue("playcnt_6h").toDouble else 0D)
          featureMap.put("playcnt_1d", if (c1.containsKey("playcnt_1d")) c1.getIntValue("playcnt_1d").toDouble else 0D)
          featureMap.put("playcnt_3d", if (c1.containsKey("playcnt_3d")) c1.getIntValue("playcnt_3d").toDouble else 0D)
          featureMap.put("playcnt_7d", if (c1.containsKey("playcnt_7d")) c1.getIntValue("playcnt_7d").toDouble else 0D)

          val d1: JSONObject = if (record.isNull("d1_feature")) new JSONObject() else
            JSON.parseObject(record.getString("d1_feature"))
          featureMap.put("return_n", if (c1.containsKey("return_n")) c1.getString("return_n").toDouble else 0D)
          featureMap.put("rovn", if (c1.containsKey("rovn")) c1.getString("rovn").toDouble else 0D)


          /*
          视频：
          视频时长、比特率

          曝光使用pv 分享使用pv 回流使用uv --> 1h 2h 3h 4h 12h 1d 3d 7d
          STR log(share) ROV log(return) ROV*log(return)
          40个特征组合
          整体、整体曝光对应、推荐非冷启root、推荐冷启root、分省份root
          200个特征值

          人：
          播放次数 --> 6h 1d 3d 7d --> 4个
          带回来的分享pv 回流uv --> 12h 1d 3d 7d --> 8个
          播放点 回流点 --> 2h 1d 3d --> 匹配数量 匹配词 语义最高相似度分 语义平均相似度分
          分享点 曝光点 （回流点） --> 1d 3d 7d 14d --> 匹配数量 匹配词 语义最高相似度分 语义平均相似度分

          头部视频：
          曝光 回流 ROVn 3个特征

          场景：
          小时 星期 apptype city province pagesource 机器型号
           */


          // b




          //4 处理label信息。
          val labels = Set(
            "pagesource", "recommend_page_type", "pagesource_change",
            "abcode",
            "is_play", "playtime",
            "is_share", "share_cnt_pv", "share_ts_list",
            "is_return", "return_cnt_pv", "return_cnt_uv", "return_mid_ts_list"
          )
          val labelNew = new JSONObject
          val labelMap = getFeatureFromSet(labels, record)
          labels.foreach(r => {
            if (labelMap.containsKey(r)) {
              labelNew.put(r, labelMap(r))
            }
          })
          //5 处理log key表头。
          val mid = record.getString("mid")
          val videoid = record.getString("videoid")
          val logtimestamp = record.getString("logtimestamp")
          val apptype = record.getString("apptype")
          val pagesource_change = record.getString("pagesource_change")
          val abcode = record.getString("abcode")
          val video_recommend = if (!record.isNull("video_recommend")) record.getString("video_recommend") else "111"

          val logKey = (mid, videoid, logtimestamp, apptype, pagesource_change, abcode, video_recommend).productIterator.mkString(":")
          val labelKey = labelNew.toString()
          val featureKey = "".toString()
          //6 拼接数据，保存。
          logKey + "\t" + labelKey + "\t" + featureKey

        })


      // 4 保存数据到hdfs
      val hdfsPath = savePath + "/" + partition
      if (hdfsPath.nonEmpty && hdfsPath.startsWith("/dw/recommend/model/")){
        println("删除路径并开始数据写入:" + hdfsPath)
        MyHdfsUtils.delete_hdfs_path(hdfsPath)
        odpsData.saveAsTextFile(hdfsPath, classOf[GzipCodec])
      }else{
        println("路径不合法，无法写入:" + hdfsPath)
      }
    }
  }

  def func(record: Record, schema: TableSchema): Record = {
    record
  }

  def getFeatureFromSet(set: Set[String], record: Record): mutable.HashMap[String, String] = {
    val result = mutable.HashMap[String, String]()
    set.foreach(r =>{
      if (!record.isNull(r)){
        try{
          result.put(r, record.getString(r))
        }catch {
          case _ => result.put(r, String.valueOf(record.getBigint(r)))
        }
      }
    })
    result
  }
}
